-------------------------------------------------------------------------------
--                                                                           --
-- File         :  commutator.vhd                                            --
--                                                                           --
-- Author(s)    : Dr. Lars Larsson (LL), www.trinamic.com                    --
--                                                                           --
-- Contributors :                                                            --
--                                                                           --
-- Description  : TMC60x commutator top level entity (from schematic)        --
--                                                                           --
-- Creation     : LL,  July 24, 2008                                         --
--                                                                           --
-- History      : LL, February 6, 2010 : BSD licensing scheme added          --
--                                                                           --
-------------------------------------------------------------------------------
-- Hint(s)      :                                                            --
-------------------------------------------------------------------------------
--
-- Copyright (c) 2010, TRINAMIC Motion Control GmbH & Co. KG 
-- All rights reserved.
--
-- Redistribution and use in source and binary forms, with or without 
-- modification, are permitted provided that the following conditions 
-- are met:
--
-- Redistributions of source code must retain the above copyright notice, 
-- this list of conditions and the following disclaimer. Redistributions 
-- in binary form must reproduce the above copyright notice, this list of 
-- conditions and the following disclaimer in the documentation and/or 
-- other materials provided with the distribution. 
--
-- Neither the name of the TRINAMIC nor the names of its contributors may 
-- be used to endorse or promote products derived from this software without 
-- specific prior written permission. 
-- 
-- THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
-- AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
-- IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
-- ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE 
-- LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
-- CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
-- SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
-- INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
-- CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
-- ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
-- THE POSSIBILITY OF SUCH DAMAGE.
--
-------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;

entity commutator is
  port (
    nRST       : in  std_logic;  -- reset 
    CLK        : in  std_logic;  -- clock
    HallFX_EN  : in  std_logic;  -- HallFX(TM) enable select, '0' : H321 / '1' : E312           
    H3, H2, H1 : in  std_logic;  -- Hall sensor inputs
    E3, E2, E1 : in  std_logic;  -- Hall sensor emulation signal (HallFX) inputs
    DualChop   : in  std_logic;  -- '0' : chopper @ HS xor LS / '1':chopper @ HS and LS
    ChopHS     : in  std_logic;  -- DualChop='1' => ChopHS='0' : chopper @ LS / ChopHS='1' : chopper @ HS            
    DIR        : in  std_logic;  -- select direction of revolution                 
    BRAKE      : in  std_logic;  -- active brake (with PWM control)               
    OVC        : in  std_logic;  -- over current ('1':disable all bridges
    PWM        : in  std_logic;  -- PWM input (from uC) to adjust speed            
    TACHO      : out std_logic;  -- tacho signal output (H3 xor H2 xor H1 resp. E3 xor E2 xor E1 )
    DIR_COM    : out std_logic;  -- measured commutation direction output (check for Hall / HallFX)             
    W_H, W_L   : out std_logic;         -- HS, LS outputs for haltf bridger W
    V_H, V_L   : out std_logic;         -- HS, LS outputs for haltf bridger W
    U_H, U_L   : out std_logic          -- HS, LS outputs for haltf bridger W
    );
end commutator;

architecture structure of commutator is

  signal w, wz, v, vz, u, uz    : std_logic;  -- intermin gate control signals
  signal wt, wb, vt, vb, ut, ub : std_logic;  -- intermin gate control signals (top, bottom)
  signal wh, wl, vh, vl, uh, ul : std_logic;  -- intermin gate control signals (high, low)

  signal pwmnovc          : std_logic;
  signal h3e3, h2e2, h1e1 : std_logic;  -- internal Hall / HallFX signals
  signal h_w, h_v, h_u    : std_logic;
  signal l_w, l_v, l_u    : std_logic;


  component chopcntl_uvw is
    port (
      DualChop       : in  std_logic;
      ChopHS         : in  std_logic;
      PWMnOVC        : in  std_logic;
      H_IN_W, L_IN_W : in  std_logic;
      H_W, L_W       : out std_logic;
      H_IN_V, L_IN_V : in  std_logic;
      H_V, L_V       : out std_logic;
      H_IN_U, L_IN_U : in  std_logic;
      H_U, L_U       : out std_logic);
  end component chopcntl_uvw;

  component dirswt is
    port (
      DIR    : in  std_logic;
      WT, WB : in  std_logic;
      WH, WL : out std_logic;
      VT, VB : in  std_logic;
      VH, VL : out std_logic;
      UT, UB : in  std_logic;
      UH, UL : out std_logic);
  end component dirswt;

  component gtcntl_uvw is
    port (
      W, WZ  : in  std_logic;
      WH, WL : out std_logic;
      V, VZ  : in  std_logic;
      VH, VL : out std_logic;
      U, UZ  : in  std_logic;
      UH, UL : out std_logic);
  end component gtcntl_uvw;

  component hall2uvw is
    port (
      H3, H2, H1          : in  std_logic;
      W, WZ, V, VZ, U, UZ : out std_logic);
  end component hall2uvw;

  component hall_chk is
    port (
      nRST     : in  std_logic;
      CLK      : in  std_logic;
      H1       : in  std_logic;
      H2       : in  std_logic;
      H3       : in  std_logic;
      STEP_OUT : out std_logic;
      DIR_OUT  : out std_logic 
      );
  end component hall_chk;

  component mux2to1x3 is
    port (
      nAB                    : in  std_logic;
      A3, B3, A2, B2, A1, B1 : in  std_logic;
      Q3, Q2, Q1             : out std_logic);
  end component mux2to1x3;

  component pwmcntl_uvw is
    port (
      BRAKE    : in  std_logic;
      PWM      : in  std_logic;
      HW, LW   : in  std_logic;
      HV, LV   : in  std_logic;
      HU, LU   : in  std_logic;
      HTW, LTW : out std_logic;
      HTV, LTV : out std_logic;
      HTU, LTU : out std_logic);
  end component pwmcntl_uvw;

begin
  
  mux2to1x3_i : mux2to1x3
      port map (
        nAB => HallFX_EN,
        A3  => H3,
        B3  => E3,
        A2  => H2,
        B2  => E2,
        A1  => H1,
        B1  => E1,
        Q3  => h3e3,
        Q2  => h2e2,
        Q1  => h1e1); 

  hall_chk_i : hall_chk
      port map (
        nRST     => nRST,
        CLK      => CLK,
        H3       => h3e3,
        H2       => h2e2,
        H1       => h1e1,
        STEP_OUT => TACHO,
        DIR_OUT  => DIR_COM);               

  hall2uvw_i : hall2uvw
      port map (
        H3 => h3e3,
        H2 => h2e2,
        H1 => h1e1,
        W  => w, WZ => wz,
        V  => v, VZ => vz,
        U  => u, UZ => uz); 

  gtcntl_uvw_i : gtcntl_uvw
      port map (
        W  => w, V => v, U => u,
        WZ => wz, Vz => vz, UZ => uz,
        WH => wt, WL => wb, VH => vt, VL => vb, UH => ut, UL => ub); 

  dirswt_i : dirswt
      port map (
        DIR => DIR,
        WT  => wt, WB => wb,
        VT  => vt, VB => vb,
        UT  => ut, UB => ub,
        WH  => wh, WL => wl,
        VH  => vh, VL => vl,
        UH  => uh, UL => ul);

  
  pwmnovc <= PWM and (not OVC);

  chopcntl_uvw_i : chopcntl_uvw
      port map (
        DualChop => DualChop,
        ChopHS   => ChopHS,
        PWMnOVC  => pwmnovc,
        H_IN_W   => wh, L_IN_W => wl, H_W => h_w, L_W => l_w,
        H_IN_V   => vh, L_IN_V => vl, H_V => h_v, L_V => l_v,
        H_IN_U   => uh, L_IN_U => ul, H_U => h_u, L_U => l_u); 

  
  pwmcntl_uvw_i : pwmcntl_uvw
      port map (
        PWM => PWM, BRAKE => BRAKE,
        HW  => h_w, LW => l_w, HTW => W_H, LTW => W_L,
        HV  => h_v, LV => l_v, HTV => V_H, LTV => V_L,
        HU  => h_u, LU => l_u, HTU => U_H, LTU => U_L); 

end structure;

