-------------------------------------------------------------------------------
--                                                                           --
-- File         : pkg_commutator_components.vhd                              --
--                                                                           --
-- Author(s)    : Dr. Lars Larsson (LL), www.trinamic.com                    --
--                                                                           --
-- Contributors :                                                            --
--                                                                           --
-- Description  : Declarations of components for TMCM-AC-840 design          --
--                                                                           --
-- Creation     : LL,    July 24, 2008                                       --
--                                                                           --
-- History      : LL, February 6, 2010 : BSD license added                   --
--                SK, May 4, 2010      : more components added/modified      --
--                                                                           --
-------------------------------------------------------------------------------
-- Hint(s)      :                                                            --
-------------------------------------------------------------------------------
--
-- Copyright (c) 2010, TRINAMIC Motion Control GmbH & Co. KG 
-- All rights reserved.
--
-- Redistribution and use in source and binary forms, with or without 
-- modification, are permitted provided that the following conditions 
-- are met:
--
-- Redistributions of source code must retain the above copyright notice, 
-- this list of conditions and the following disclaimer. Redistributions 
-- in binary form must reproduce the above copyright notice, this list of 
-- conditions and the following disclaimer in the documentation and/or 
-- other materials provided with the distribution. 
--
-- Neither the name of the TRINAMIC nor the names of its contributors may 
-- be used to endorse or promote products derived from this software without 
-- specific prior written permission. 
-- 
-- THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
-- AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
-- IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
-- ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE 
-- LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
-- CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
-- SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
-- INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
-- CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
-- ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
-- THE POSSIBILITY OF SUCH DAMAGE.
--
-------------------------------------------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

package pkg_commutator_components is

  constant CTRL_REG_0_ADDR : std_logic_vector(23 downto 0) := x"0F0000";
  constant CTRL_REG_1_ADDR : std_logic_vector(23 downto 0) := x"0F0004";
  constant CTRL_REG_2_ADDR : std_logic_vector(23 downto 0) := x"0F0008";
  constant CTRL_REG_3_ADDR : std_logic_vector(23 downto 0) := x"0F000C";
  constant CTRL_REG_4_ADDR : std_logic_vector(23 downto 0) := x"0F0010";
  constant CTRL_REG_5_ADDR : std_logic_vector(23 downto 0) := x"0F0014";

  constant TMC603_PWM_VAL_REG_ADDR   : std_logic_vector(23 downto 0) := x"0F0018";
  constant TMC262_WC_REG_ADDR        : std_logic_vector(23 downto 0) := x"0F001C";
  constant TMC262_MR_REG_ADDR        : std_logic_vector(23 downto 0) := x"0F0020";
  constant TMC262_SPEED_VAL_REG_ADDR : std_logic_vector(23 downto 0) := x"0F0024";


  component tmc603_commutator_top
    port (
      CLK          : in  std_logic;
      nRST         : in  std_logic;
      LED1_4       : out std_logic_vector(3 downto 0);
      PB1          : in  std_logic;
      PB2          : in  std_logic;
      PB3          : in  std_logic;
      PB4          : in  std_logic;
      PB5          : in  std_logic;
      BL1          : out std_logic;
      BH1          : out std_logic;
      BH2          : out std_logic;
      BL2          : out std_logic;
      BH3          : out std_logic;
      BL3          : out std_logic;
      HALL1        : in  std_logic;
      HALL2        : in  std_logic;
      HALL3        : in  std_logic;
      HALLFX1      : in  std_logic;
      HALLFX2      : in  std_logic;
      HALLFX3      : in  std_logic;
      ENABLE_TRI_D : out std_logic;
      ENABLE_TRI_E : out std_logic;
      SAMPLEx      : out std_logic;
      SENSE_HI     : out std_logic;
      SCCLK        : out std_logic;
      CLR_ERR      : out std_logic;
      USER_ENN     : in  std_logic;
      U_RD_DATA    : out std_logic_vector(31 downto 0));
  end component;

  component tmc262_commutator_top
    
    port (
      CLK          : in  std_logic;
      nRST         : in  std_logic;
      LED1_4       : out std_logic_vector(3 downto 0);
      PB1          : in  std_logic;
      PB2          : in  std_logic;
      PB3          : in  std_logic;
      PB4          : in  std_logic;
      PB5          : in  std_logic;
      CLK_262      : out std_logic;
      SCK_SPI      : out std_logic;
      CSN_SPI      : out std_logic;
      SDO_SPI      : in  std_logic;
      SDI_SPI      : out std_logic;
      DIR          : out std_logic;
      STEP         : out std_logic;
      SG_TST       : in  std_logic;
      ENN          : out std_logic;
      USER_ENN     : in  std_logic;
      MRES_SEL     : in  std_logic;
      U_RD_DATA_WC : out std_logic_vector(31 downto 0);
      U_RD_DATA_MR : out std_logic_vector(31 downto 0);
      U_RD_DATA_SV : out std_logic_vector(31 downto 0));
  end component;

  component bridge_APB3_to_userlogic
    port (
      PCLK              : in  std_logic;
      PRESETn           : in  std_logic;
      PENABLE           : in  std_logic;
      PSEL              : in  std_logic;
      PADDR             : in  std_logic_vector(23 downto 0);
      PWRITE            : in  std_logic;
      PWDATA            : in  std_logic_vector(31 downto 0);
      PRDATA            : out std_logic_vector(31 downto 0);
      PREADY            : out std_logic;
      PSLVERR           : out std_logic;
      U_WR_REQ          : out std_logic;  -- notify a write from bus to userlogic
      U_WR_ACK          : in  std_logic;  -- acknowledge that user side has received the data (U_WR_DATA)
      U_WR_DATA         : out std_logic_vector(31 downto 0);
      U_WR_ADDR         : out std_logic_vector(23 downto 0);
      CTRL_REG_0        : in  std_logic_vector(31 downto 0);
      CTRL_REG_1        : in  std_logic_vector(31 downto 0);
      CTRL_REG_2        : in  std_logic_vector(31 downto 0);
      CTRL_REG_3        : in  std_logic_vector(31 downto 0);
      CTRL_REG_4        : in  std_logic_vector(31 downto 0);
      CTRL_REG_5        : in  std_logic_vector(31 downto 0);
      U_RD_DATA_603_PWM : in  std_logic_vector(31 downto 0);
      U_RD_DATA_262_WC  : in  std_logic_vector(31 downto 0);
      U_RD_DATA_262_MR  : in  std_logic_vector(31 downto 0);
      U_RD_DATA_262_SV  : in  std_logic_vector(31 downto 0));
  end component;

  component tmc603_commutator
    port(
      nRST              : in  std_logic;  -- high active asynchronous reset 
      CLK               : in  std_logic;  -- clock input                           
      HallFX_EN         : in  std_logic;  -- HallFX(TM) enable select, '0' : H321 / '1' : E312 (hallFX)                           
      DualChop          : in  std_logic;  -- '0' : chopper @ HS xor LS / '1':chopper @ HS and LS
      ChopHS            : in  std_logic;  -- DualChop='1' => ChopHS='0' : chopper @ LS / ChopHS='1' : chopper @ HS
                                          -- hallFX needs DualChop='1' and ChopHS='0'                            
      OVC               : in  std_logic;  -- over current ('1':disable all bridges (can be handled by TMC603)
      PWM               : in  std_logic;  -- PWM input (e.g. from uC) to adjust speed via duty cycle                           
      DIR               : in  std_logic;  -- select direction of revolution / commutation                           
      BRAKE             : in  std_logic;  -- active brake (with PWM control)                           
      TACHO             : out std_logic;  -- tacho signal output (H3 xor H2 xor H1 resp. E3 xor E2 xor E1 )
      DIR_COM           : out std_logic;  -- measured commutation direction output (check for Hall / HallFX)
                                          -- '0' vs. '1' needs to be associated to 'cw' vs. 'ccw' rotation                              
      TMC603_BBM_EN     : out std_logic;  -- enables internal break-before-make circuitry of TMC603
      TMC603_ENABLE     : out std_logic;  -- '0' : drivers disabled / '1' : bridges enabled                           
      TMC603_BH1        : out std_logic;  -- high side gate control, bridge 1 (high active)
      TMC603_BH2        : out std_logic;  -- high side gate control, bridge 2 (high active)
      TMC603_BH3        : out std_logic;  -- high side gate control, bridge 3 (high active)                               
      TMC603_BL1        : out std_logic;  -- low side gate control, bridge 1 (high active)
      TMC603_BL2        : out std_logic;  -- low side gate control, bridge 2 (high active)
      TMC603_BL3        : out std_logic;  -- low side gate control, bridge 3 (high active)                           
      TMC603_SAMPLE_123 : out std_logic;  -- common sample trigger signal for TMC603 inputs SAMPLE1, SAMPLE2, SAMPLE3                           
      TMC603_SCCLK      : out std_logic;  -- switched capacitor filter clock signal for TMC603 hallFX                           
      TMC603_CLR_ERR    : out std_logic;  -- clear error condition flag of TMC603 
      TMC603_ERROR_OUT  : in  std_logic;  -- error output of TMC603 (can be dicertly connected to CLR_ERR of TMC603                           
      TMC603_SENSE_HI   : out std_logic;  -- '0':low sense amplification / '1':high sense amplification of TMC603
      TMC603_H1         : in  std_logic;  -- hallXF output of Hall Sensor emulation signal H1
      TMC603_H2         : in  std_logic;  -- hallXF output of Hall Sensor emulation signal H2
      TMC603_H3         : in  std_logic;  -- hallXF output of Hall Sensor emulation signal H3
      HALL1             : in  std_logic;  -- Hall Signal input 1
      HALL2             : in  std_logic;  -- Hall Signal input 2
      HALL3             : in  std_logic   -- Hall Signal input 3
      );
  end component;

  component commutator
    port (
      nRST       : in  std_logic;       -- reset
      CLK        : in  std_logic;       -- clock
      HallFX_EN  : in  std_logic;       -- HallFX(TM) enable select
      H3, H2, H1 : in  std_logic;       -- Hall sensor inputs
      E3, E2, E1 : in  std_logic;  -- Hall sensor emulation signal (HallFX) inputs
      DualChop   : in  std_logic;  -- '0' : chopper @ HS xor LS / '1':chopper @ HS and LS
      ChopHS     : in  std_logic;  -- DualChop='1' => ChopHS='0' : chopper @ LS / ChopHS='1' : chopper @ HS
      DIR        : in  std_logic;       -- select direction of revolution
      BRAKE      : in  std_logic;       -- active brake (with PWM control)
      OVC        : in  std_logic;  -- over current ('1':disable all bridges
      PWM        : in  std_logic;       -- PWM input (from uC) to adjust speed
      TACHO      : out std_logic;  -- tacho signal output (H3 xor H2 xor H1 resp. E3 xor E2 xor E1 )
      DIR_COM    : out std_logic;  -- measured commutation direction output (check for Hall / HallFX)
      W_H, W_L   : out std_logic;       -- HS, LS outputs for haltf bridger W
      V_H, V_L   : out std_logic;       -- HS, LS outputs for haltf bridger W
      U_H, U_L   : out std_logic        -- HS, LS outputs for haltf bridger W
      );     
  end component;

  component mux2to1x3
    port (
      nAB                    : in  std_logic;
      A3, B3, A2, B2, A1, B1 : in  std_logic;
      Q3, Q2, Q1             : out std_logic);
  end component;

  component hall_chk
    port (
      nRST     : in  std_logic;
      CLK      : in  std_logic;
      H1       : in  std_logic;
      H2       : in  std_logic;
      H3       : in  std_logic;
      STEP_OUT : out std_logic;
      DIR_OUT  : out std_logic
      );
  end component;

  component hall2uvw
    port (
      H3, H2, H1          : in  std_logic;
      W, WZ, V, VZ, U, UZ : out std_logic);
  end component;


  component gtcntl_uvw
    port (
      W, WZ  : in  std_logic;
      WH, WL : out std_logic;
      V, VZ  : in  std_logic;
      VH, VL : out std_logic;
      U, UZ  : in  std_logic;
      UH, UL : out std_logic);
  end component;

  component dirswt
    port (

      DIR    : in  std_logic;
      WT, WB : in  std_logic;
      WH, WL : out std_logic;
      VT, VB : in  std_logic;
      VH, VL : out std_logic;
      UT, UB : in  std_logic;
      UH, UL : out std_logic);
  end component;

  component chopcntl_uvw
    port (
      DualChop       : in  std_logic;
      ChopHS         : in  std_logic;
      PWMnOVC        : in  std_logic;
      H_IN_W, L_IN_W : in  std_logic;
      H_W, L_W       : out std_logic;
      H_IN_V, L_IN_V : in  std_logic;
      H_V, L_V       : out std_logic;
      H_IN_U, L_IN_U : in  std_logic;
      H_U, L_U       : out std_logic);
  end component;

  component pwmcntl_uvw
    port (
      BRAKE    : in  std_logic;
      PWM      : in  std_logic;
      HW, LW   : in  std_logic;
      HV, LV   : in  std_logic;
      HU, LU   : in  std_logic;
      HTW, LTW : out std_logic;
      HTV, LTV : out std_logic;
      HTU, LTU : out std_logic);
  end component;

  component spi_txrx_TMC262
    port (
      nRST      : in  std_logic;
      CLK       : in  std_logic;
      SEND_DATA : in  std_logic_vector(19 downto 0);
      RECV_DATA : out std_logic_vector(19 downto 0);
      READY     : out boolean;
      START     : in  std_logic;
      SCK       : out std_logic;
      CSN       : out std_logic;
      SDI       : out std_logic;
      SDO       : in  std_logic);
  end component;

  -- conversions
  function to_unsigned_std_logic_vector (arg : integer; size : integer) return std_logic_vector;
  function to_signed_std_logic_vector (arg   : integer; size : integer) return std_logic_vector;
  function to_unsigned_integer (arg          : std_logic_vector) return integer;
  function to_signed_integer (arg            : std_logic_vector) return integer;
  
end;

package body pkg_commutator_components is

  function to_unsigned_std_logic_vector (arg : integer; size : integer) return std_logic_vector is
  begin
    return std_logic_vector(to_unsigned(arg, size));
  end to_unsigned_std_logic_vector;

  function to_signed_std_logic_vector (arg : integer; size : integer) return std_logic_vector is
  begin
    return std_logic_vector(to_signed(arg, size));
  end to_signed_std_logic_vector;

  function to_unsigned_integer (arg : std_logic_vector) return integer is
  begin
    return to_integer(unsigned(arg));
  end to_unsigned_integer;

  function to_signed_integer (arg : std_logic_vector) return integer is
  begin
    return to_integer(signed(arg));
  end to_signed_integer;

end pkg_commutator_components;
