-------------------------------------------------------------------------------
--                                                                           --
-- File         : tmc262_commutator_top.vhd                                  --
--                                                                           --
-- Author(s)    : Dr. Stephan Kubisch (SK), www.trinamic.com                 --
--                                                                           --
-- Contributors :                                                            --
--                                                                           --
-- Description  : Top level design for the TMC262 commutation logic          --
--                using the SPI-interface for configuration and step/dir
--                interface for motor commutation.
--                                                                           --
--                                                                           --
-- Creation     : SK, April 9, 2010                                          --
--                                                                           --
-- History      : SK, April 12, 2010 - Added process for dir and step strobe
--                SK, May 4, 2010    - fixed some SPI problems
--                SK, May 17, 2010   - Added MOSFETs on/off via SPI
--                SK, May 28, 2010   - fixed error in wait_cycle calculation
--                                   - added example SPI datagrams
--                                   - added microstep resolution control
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
--
-- Copyright (c) 2010, TRINAMIC Motion Control GmbH & Co. KG 
-- All rights reserved.
--
-- Redistribution and use in source and binary forms, with or without 
-- modification, are permitted provided that the following conditions 
-- are met:
--
-- Redistributions of source code must retain the above copyright notice, 
-- this list of conditions and the following disclaimer. Redistributions 
-- in binary form must reproduce the above copyright notice, this list of 
-- conditions and the following disclaimer in the documentation and/or 
-- other materials provided with the distribution. 
--
-- Neither the name of the TRINAMIC nor the names of its contributors may 
-- be used to endorse or promote products derived from this software without 
-- specific prior written permission. 
-- 
-- THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
-- AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
-- IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
-- ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE 
-- LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
-- CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
-- SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
-- INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
-- CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
-- ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
-- THE POSSIBILITY OF SUCH DAMAGE.
--
-------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.numeric_std.all;

use work.pkg_commutator_components.all;

entity tmc262_commutator_top is
  
  port (
    CLK             : in  std_logic;
    nRST            : in  std_logic;
    PB1             : in  std_logic;  -- speed UP
    PB2             : in  std_logic;  -- change direction/step resolution
    PB3             : in  std_logic;  -- trigger SPI config
    PB4             : in  std_logic;  -- change direction/step resolution
    PB5             : in  std_logic;  -- speed down
    LED1_4          : out std_logic_vector(3 downto 0);
    CLK_262         : out std_logic;  -- system clock for TMC262 delivered by FPGA (20MHz)
    SCK_SPI         : out std_logic;  -- SPI clk delivered by SPI master (FPGA) to TMC262 (10MHz)
    CSN_SPI         : out std_logic;  -- SPI chip enable 
    SDO_SPI         : in  std_logic;  -- SPI data from the TMC262
    SDI_SPI         : out std_logic;  -- SPI data to the TMC262
    DIR             : out std_logic;  -- direction for step/dir mode
    STEP            : out std_logic;  -- step for step/dir mode
    SG_TST          : in  std_logic;  -- signals motor stall (active high), stallguard2 (TM)
    ENN             : out std_logic;  -- not enable MOSFET brigdes
    USER_ENN        : in  std_logic;  -- enable/disable TMC262 by clocking/not clocking the IC
    MRES_SEL        : in  std_logic;  -- switches speed/MRES control via PB2 and PB4
    U_RD_DATA_WC    : out std_logic_vector(31 downto 0);
    U_RD_DATA_MR    : out std_logic_vector(31 downto 0);
    U_RD_DATA_SV    : out std_logic_vector(31 downto 0)
    );
 
end tmc262_commutator_top;

architecture structure of tmc262_commutator_top is

  type spi_cntrl_state_type is (idle,
                                config1,
                                config2,
                                config3,
                                config4,
                                config5,
                                config1_wait,
                                config2_wait,
                                config3_wait,
                                config4_wait
                                );

  signal spi_cntrl_state_ff  : spi_cntrl_state_type  := idle;

  signal DIR_ff, step_strobe_ff, ENN_ff   : std_logic;
  signal SPI_START                        : std_logic;
  signal SPI_READY                        : boolean;
  signal SEND_DATA_ff, RECV_DATA_ff       : std_logic_vector(19 downto 0);
  signal wait_cycles_ff                   : std_logic_vector(19 downto 0);
  signal speed_ff                         : std_logic_vector(7  downto 0);
  signal step_cntr_ff                     : std_logic_vector(19 downto 0);
  signal divider_ff                       : std_logic_vector(19 downto 0);
  signal DS3_previous_ff                  : std_logic;
  signal mres_ff,mres_prog_ff             : std_logic_vector(3 downto 0);
  signal PB2_old_ff, PB4_old_ff           : std_logic;
  
  -- factor: 1 = much much faster (take care), 2 = medium, 3 = slower
  constant factor                         : integer range 1 to 3 := 2;

begin

  -- -------------------------------------------------------------------------
  -- BEGIN of mapping of internal signals ------------------------------------
  -- -------------------------------------------------------------------------
  CLK_262                 <= CLK;
  U_RD_DATA_WC(19 downto 0) <= wait_cycles_ff;
  U_RD_DATA_MR(3  downto 0) <= mres_ff;
  U_RD_DATA_SV(7  downto 0) <= speed_ff;              -- actual "speed" value
  DIR                     <= DIR_ff;
  STEP                    <= step_strobe_ff;
  ENN                     <= ENN_ff;

  LED1_4(0) <= speed_ff(0);
  LED1_4(1) <= speed_ff(1);
  LED1_4(2) <= speed_ff(2);
  LED1_4(3) <= speed_ff(3);


  -----------------------------------------------------------------------------
  -- controls the enable flipflop
  --
  -- Enables the MOSFET bridges of the TMC262
  -----------------------------------------------------------------------------
  enn_p : process(CLK, nRST)
  begin
    if nRST = '0' then
      
      ENN_ff <= '0';
      
    elsif rising_edge(CLK) then

      if USER_ENN = '1' then
        ENN_ff <= '0';    -- ENN = not enable! so it is low acive
      else
        ENN_ff <= '1';       
      end if;
      
    end if;
  end process;

  -----------------------------------------------------------------------------
  -- controls the direction flipflop
  --
  -- Depending on the actual motor speed, the microstep resolution and the
  -- motor type, direction change might lead to a motor stall as no ramps are
  -- used here before changing the direction (it is a hard stop).
  -- It is tested with 256 microstep resolution at various speeds.
  -----------------------------------------------------------------------------
  dir_p : process(CLK, nRST)
  begin
    if nRST = '0' then
      
      DIR_ff <= '0';
      
    elsif rising_edge(CLK) then

      if MRES_SEL = '0' then
        if PB2 = '1' then
          DIR_ff <= '1';
        elsif PB4 = '1' then
          DIR_ff <= '0';
        end if;
      end if;
      
    end if;
  end process;

  -----------------------------------------------------------------------------
  -- controls the microstep resolution
  -----------------------------------------------------------------------------
  mres_p : process(CLK, nRST)
  begin
    if nRST = '0' then
      
      PB2_old_ff <= '0';
      PB4_old_ff <= '0';
      mres_ff <= "0000";       -- 256 microsteps
--      mres_ff <= "0001";       -- 128 microsteps
--      mres_ff <= "0010";       --  64 microsteps
--      mres_ff <= "0011";       --  32 microsteps
--      mres_ff <= "0100";       --  16 microsteps
--      mres_ff <= "0101";       --   8 microsteps
--      mres_ff <= "0110";       --   4 microsteps
--      mres_ff <= "0111";       --   2 microsteps
--      mres_ff <= "1000";       --   fullstepping

    elsif rising_edge(CLK) then

      PB2_old_ff <= PB2;
      PB4_old_ff <= PB4;
      if MRES_SEL = '1' then
        -- change only on rising edge of the push button to not overshoot
        -- decrease resolution
        if (PB2 = '1' and PB2_old_ff = '0') then
          if mres_ff < "1000" then
            mres_ff <= mres_ff + 1;
          end if;
        -- increase resolution
        elsif (PB4 = '1' and PB4_old_ff = '0') then
          if mres_ff > "0000" then
            mres_ff <= mres_ff - 1;
          end if;
        end if;
      end if;
      
    end if;
  end process;

  -----------------------------------------------------------------------------
  -- SPI control
  -- todo MRES change
  -----------------------------------------------------------------------------
  spi_cntrl_p : process(CLK, nRST)
  begin
    if nRST = '0' then
      
      spi_cntrl_state_ff <= idle;
      SEND_DATA_ff       <= x"00000";   -- 20bit to send via SPI
      SPI_START          <= '0';        -- set to start tx SEND_DATA
      DS3_previous_ff    <= '0';
      mres_prog_ff       <= "0000";
      
    elsif rising_edge(CLK) then

      -- statemachine for SPI configuration and control
      -- numbers indicate different priorities:
      case spi_cntrl_state_ff is
        when idle =>
          --  always when PB3 is triggered --> repeat configuration
          if PB3 = '1' then
            if SPI_READY then
              -- compose config datagram 1/5: 
              SEND_DATA_ff       <= x"EF040";  -- DRVCONF (max. driver strength, step/dir mode, Vsense = '1' => VFS = 1/32 Vdd)
              SPI_START          <= '1';
              spi_cntrl_state_ff <= config1;
              DS3_previous_ff    <= '1';
            else
              SEND_DATA_ff       <= x"00000";
              SPI_START          <= '0';
              spi_cntrl_state_ff <= idle;
            end if;
          else
            SEND_DATA_ff       <= x"00000";
            SPI_START          <= '0';
            spi_cntrl_state_ff <= idle;
          end if;
        when config1 =>
          SPI_START          <= '0';
          spi_cntrl_state_ff <= config1_wait;
        when config1_wait =>
          -- wait until ready again
          if SPI_READY then
            -- compose config datagram 2/5: 
            SEND_DATA_ff       <= x"D7F0F";  -- SGCSCONF (SG = 7F = low sensivity, CS = 31 --> max. current scale res)
            SPI_START          <= '1';
            spi_cntrl_state_ff <= config2;
          else
            SEND_DATA_ff       <= x"00000";
            SPI_START          <= '0';
            spi_cntrl_state_ff <= config1_wait;
          end if;
        when config2 =>
          SPI_START          <= '0';
          spi_cntrl_state_ff <= config2_wait;
        when config2_wait =>
          -- wait until ready again
          if SPI_READY then
            -- compose config datagram 3/5: 
            SEND_DATA_ff       <= x"901B4";  -- CHOPCONF (Hysteresis mode, hysterese value = "1011" = 8, bridges on)
            SPI_START          <= '1';
            spi_cntrl_state_ff <= config3;
          else
            SEND_DATA_ff       <= x"00000";
            SPI_START          <= '0';
            spi_cntrl_state_ff <= config2_wait;
          end if;
        when config3 =>
          SPI_START          <= '0';
          spi_cntrl_state_ff <= config3_wait;
        when config3_wait =>
          -- wait until ready again
          if SPI_READY then
            -- compose config datagram 4/5: 
            SEND_DATA_ff       <= x"A8202";  -- SMARTEN (test of coolStep current control)
            SPI_START          <= '1';
            spi_cntrl_state_ff <= config4;
          else
            SEND_DATA_ff       <= x"00000";
            SPI_START          <= '0';
            spi_cntrl_state_ff <= config3_wait;
          end if;
        when config4 =>
          SPI_START          <= '0';
          spi_cntrl_state_ff <= config4_wait;   
        when config4_wait =>
          -- wait until ready again
          if SPI_READY then
            -- compose config datagram 5/5:
            SEND_DATA_ff       <= x"0000" & mres_ff;  -- DRVCTRL (no INTPOL, no DEDGE, microstep resolution)
            SPI_START          <= '1';
            mres_prog_ff       <= mres_ff;  -- now the user controlled resolution is configured
            spi_cntrl_state_ff <= config5;
          else
            SEND_DATA_ff       <= x"00000";
            SPI_START          <= '0';
            spi_cntrl_state_ff <= config4_wait;
          end if;
        when config5 =>
          SPI_START          <= '0';
          spi_cntrl_state_ff <= idle;
        when others => null;
      end case;

   ----------------------------------------------------------------------------
   -- SPI Example Datagrams
   -- (see the TMC262 Datasheet for documentation on the SPI config registers)
   ----------------------------------------------------------------------------
   -- --- Main operation mode registers -------------------
   --    CHOPCONF = x"901B4"; // Hysteresis mode,, MOSFET bridges turned ON
   -- or CHOPCONF = x"901B0"; // Hysteresis mode,, MOSFET bridges turned OFF
   -- or CHOPCONF = x"94557"; // Const. t_off mode, MOSFET bridges turned ON
   -- or CHOPCONF = x"94550"; // Const. t_off mode, MOSFET bridges turned OFF
      
   --    SGCSCONF = x"D000A"; // Current setting: (x"d001F" = max. current)
      
   --    DRVCONF  = x"EF090"; // max. driver strength HS/LS, stallGuard read back, sdoff=1=SPI mode
   -- or DRVCONF  = x"EF000"; // max. driver strength HS/LS, microstep pos read back, sdoff=0=step/dir mode
      
   --    SMARTEN  = x"A8202"; // test of coolStep current control
      
   --    DRVCTRL  = "00" & Phase_A & Current_A & Phase_B & Current_B   (for SPI mode)
   -- or DRVCTRL  = x"00000" = no INTPOL, no DEDGE, 256 Microsteps/Fullstep  (for step/dir mode)
   ----------------------------------------------------------------------------
      
    end if;    
  end process;

  -----------------------------------------------------------------------------
  -- Speed Control
  -- TODO:
  --
  -- modify wait cycle calculation together with microstep resolution to keep
  -- same speed
  --    increase wait cycle count with lower resolution
  --    decrease wait cycle count with higher resolution
  -----------------------------------------------------------------------------
  updn_p : process (nRST, CLK)
  begin
    if nRST = '0' then
      
      wait_cycles_ff <= (others => '1');
      speed_ff       <= (others => '0');
      divider_ff     <= (others => '0');
      step_cntr_ff   <= (others => '0');
      step_strobe_ff <= '0';
      
    elsif rising_edge(CLK) then

      -- manage speed using wait cycles      
      divider_ff <= divider_ff + 1;
      if divider_ff = x"00000" then  -- inc/dec only @ 00000, so that you can "feel" it        
        if PB1 = '1' then               -- speed up 
          if speed_ff < x"FF" then
            speed_ff <= speed_ff + 1;
          end if;
        elsif PB5 = '1' then            -- slow down
          if speed_ff > x"00" then
            speed_ff <= speed_ff - 1;
          end if;
        end if;
      end if;

      -- manage step strobe signal
      -- @ speed_ff = x"FF" speed should be 0, so no strobe will be set at all
      if speed_ff > x"00" then
        
        -- reload counter for next strobe & set strobe at counter = 0
        if step_cntr_ff = x"00000" then
          step_strobe_ff <= '1';             -- set strobe signal
          step_cntr_ff   <= wait_cycles_ff;
          
        -- Set the strobe already one cycles before, so pulse width is 2 clock periods
        -- As the TMC262 is clocked at its max. frequency and also has an
        -- additional glitch filter at the step input it is more save to use pulse
        -- width of 2 clock periods.
        elsif step_cntr_ff = x"00001" then -- and step_cntr_ff /= x"00000" then
          step_strobe_ff <= '1';             -- set strobe signal
          step_cntr_ff   <= step_cntr_ff - 1;

        -- no strobe signal set, just decrement counter
        else
          step_strobe_ff <= '0';
          step_cntr_ff   <= step_cntr_ff - 1;
        end if;
        
      else
        step_strobe_ff <= '0';
      end if;

      -- manage wait cycles
      -- wait_cycles_ff defines the clock cycles to count until next step strobe is set
      -- include microstep resolution mres_ff here as well
      -- the mres_prog_ff holds the value that is configured via SPI
      --    mres_prog_ff is x"0" for 256 microsteps
      --    mres_prog_ff is x"1" for 128 microsteps
      --      ...
      --    mres_prog_ff is x"8" for fullstepping
      --    --> Accordingly, the number if wait cycles needs to be doubled each
      --    time the resolution is reduced to keep approximately the same velocity.
      --    It needs to be halved each time the resolution is increased.
      --    This is necessary especially for high velocities to not accidently
      --    double the commutation frequency over the motor's or the application's maximum rpm value.
      if speed_ff = x"00" then
        wait_cycles_ff <= x"FFFFF";
      elsif speed_ff >= x"F0" then      -- x"F0" = 240, fixed value to limit motor speed to a resonable :-) value
        wait_cycles_ff <= to_unsigned_std_logic_vector(((255-240)*factor*(2**to_unsigned_integer(mres_prog_ff))),20);
      else       
        wait_cycles_ff <= to_unsigned_std_logic_vector(((255-to_unsigned_integer(speed_ff))*factor*(2**(to_unsigned_integer(mres_prog_ff)))),20);
      end if;
      
    end if;  -- rising_edge
  end process;

  
  -----------------------------------------------------------------------------
  -- SPI-Interface Module
  -----------------------------------------------------------------------------
  -- SPI CLK is half the system clock (20/2 = 10 MHz)
  spi_txrx_TMC262_1: spi_txrx_TMC262
    port map (
      nRST      => nRST,
      CLK       => CLK,
      SEND_DATA => SEND_DATA_ff,
      RECV_DATA => RECV_DATA_ff,
      READY     => SPI_READY,
      START     => SPI_START,
      SCK       => SCK_SPI,
      CSN       => CSN_SPI,
      SDI       => SDI_SPI,
      SDO       => SDO_SPI);

  
end structure;
