-------------------------------------------------------------------------------
--                                                                           --
-- File         : tmc603_commutator_top.vhd                                  --
--                                                                           --
-- Author(s)    : Dr. Lars Larsson (LL), www.trinamic.com                    --
--                                                                           --
-- Contributors :                                                            --
--                                                                           --
-- Description  : Top level design for the TMC603A commutation logic for the --
--                Trinamic TMCM-AC-840 Motor Control Daughter Card Kit       --
--                used with the Actel SmartFusion Development Kit            --
--                                                                           --                                                                           --
-- Creation     : LL, February 15, 2010                                      --
--                                                                           --
-- History      : LL, February 20, 2010 : set DualChop='1' (for non-hallFX)  --
--                LL, February 20, 2010 : hint conerning disconnection of    --
--                                        power supply during programming    --
--                SK, March 09, 2010    : modified for new bridge module
--                                        added component and logic for
--                                        bridge_AHBlite_APB3_to_userlogic
--                SK, March 11, 2010    : DIP-switches moved to MSS 
--                SK, April 22, 2010    : Synchronisation modified           --
--                SK, May 4, 2010       : Synch moved to top level           -- 
--                                                                           --
-------------------------------------------------------------------------------
--                                                                           --
-- Hint(s) : *** DISCONNECT BLDC motor power supply during programming ***   --
--                                                                           --
-------------------------------------------------------------------------------
--
-- Copyright (c) 2010, TRINAMIC Motion Control GmbH & Co. KG 
-- All rights reserved.
--
-- Redistribution and use in source and binary forms, with or without 
-- modification, are permitted provided that the following conditions 
-- are met:
--
-- Redistributions of source code must retain the above copyright notice, 
-- this list of conditions and the following disclaimer. Redistributions 
-- in binary form must reproduce the above copyright notice, this list of 
-- conditions and the following disclaimer in the documentation and/or 
-- other materials provided with the distribution. 
--
-- Neither the name of the TRINAMIC nor the names of its contributors may 
-- be used to endorse or promote products derived from this software without 
-- specific prior written permission. 
-- 
-- THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
-- AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
-- IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
-- ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE 
-- LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
-- CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
-- SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
-- INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
-- CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
-- ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
-- THE POSSIBILITY OF SUCH DAMAGE.
--
-------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

use work.pkg_commutator_components.all;

entity tmc603_commutator_top is
  
  port (
    CLK             : in  std_logic;
    nRST            : in  std_logic;
    LED1_4          : out std_logic_vector(3 downto 0);
    PB1             : in  std_logic;  -- speed UP
    PB2             : in  std_logic;  -- direction left
    PB3             : in  std_logic;  -- samplex
    PB4             : in  std_logic;  -- direction right
    PB5             : in  std_logic;  -- speed down
    BL1             : out std_logic;
    BH1             : out std_logic;
    BH2             : out std_logic;
    BL2             : out std_logic;
    BH3             : out std_logic;
    BL3             : out std_logic;
    HALL1           : in  std_logic;
    HALL2           : in  std_logic;
    HALL3           : in  std_logic;
    HALLFX1         : in  std_logic;
    HALLFX2         : in  std_logic;
    HALLFX3         : in  std_logic;
    ENABLE_TRI_D    : out std_logic;
    ENABLE_TRI_E    : out std_logic;
    SAMPLEx         : out std_logic;
    SENSE_HI        : out std_logic;
    SCCLK           : out std_logic;
    CLR_ERR         : out std_logic;
    USER_ENN        : in  std_logic;
    U_RD_DATA       : out std_logic_vector(31 downto 0));

end tmc603_commutator_top;

architecture structure of tmc603_commutator_top is

  signal pwm, pwm_clr_ff, dir_last_ff, dir_ff, tacho, dir_com       : std_logic;
  signal hall1_ff, hall2_ff, hall3_ff                               : std_logic;
  signal CLR_ERR_ff, SAMPLEx_ff                                     : std_logic;
  signal ENABLE_TRI_E_ff, ENABLE_TRI_D_ff                           : std_logic;

  signal pwm_val_o : std_logic_vector (7 downto 0);
  signal pwm_val_wr           : std_logic;                    

  signal CoreWrEn : std_logic;
  signal CoreRdEn : std_logic;

  component tmc603_commutator is
    port (
      nRST              : in  std_logic;  -- high active asynchronous reset 
      CLK               : in  std_logic;  -- clock input
      HallFX_EN         : in  std_logic;  -- HallFX(TM) enable select, '0' : H321 / '1' : E312                   
      DualChop          : in  std_logic;  -- '0' : chopper @ HS xor LS / '1':chopper @ HS and LS
      ChopHS            : in  std_logic;  -- DualChop='1' => ChopHS='0' : chopper @ LS / ChopHS='1' : chopper @ HS
                                          -- hallFX needs DualChop='1' and ChopHS='0'                          
      OVC               : in  std_logic;  -- over current ('1':disable all bridges (can be handled by TMC603)
      PWM               : in  std_logic;  -- PWM input (e.g. from uC) to adjust speed via duty cycle                           
      DIR               : in  std_logic;  -- select direction of revolution / commutation                           
      BRAKE             : in  std_logic;  -- active brake (with PWM control)                           
      TACHO             : out std_logic;  -- tacho signal output (H3 xor H2 xor H1 resp. E3 xor E2 xor E1 )
      DIR_COM           : out std_logic;  -- measured commutation direction output (check for Hall / HallFX)
                                          -- '0' vs. '1' needs to be associated to 'cw' vs. 'ccw' rotation                              
      TMC603_BBM_EN     : out std_logic;  -- enables internal break-before-make circuitry of TMC603
      TMC603_ENABLE     : out std_logic;  -- '0' : drivers disabled / '1' : bridges enabled                           
      TMC603_BH1        : out std_logic;  -- high side gate control, bridge 1 (high active)
      TMC603_BH2        : out std_logic;  -- high side gate control, bridge 2 (high active)
      TMC603_BH3        : out std_logic;  -- high side gate control, bridge 3 (high active)                               
      TMC603_BL1        : out std_logic;  -- low side gate control, bridge 1 (high active)
      TMC603_BL2        : out std_logic;  -- low side gate control, bridge 2 (high active)
      TMC603_BL3        : out std_logic;  -- low side gate control, bridge 3 (high active)                           
      TMC603_SAMPLE_123 : out std_logic;  -- common sample trigger signal for TMC603 inputs SAMPLE1, SAMPLE2, SAMPLE3                           
      TMC603_SCCLK      : out std_logic;  -- switched capacitor filter clock signal for TMC603 hallFX                           
      TMC603_CLR_ERR    : out std_logic;  -- clear error condition flag of TMC603 
      TMC603_ERROR_OUT  : in  std_logic;  -- error output of TMC603 (can be dicertly connected to CLR_ERR of TMC603                           
      TMC603_SENSE_HI   : out std_logic;  -- '0':low sense amplification / '1':high sense amplification of TMC603                 
      TMC603_H1         : in  std_logic;  -- hallXF output of TMC603A Hall Sensor emulation signal H1
      TMC603_H2         : in  std_logic;  -- hallXF output of TMC603A Hall Sensor emulation signal H2
      TMC603_H3         : in  std_logic;  -- hallXF output of TMC603A Hall Sensor emulation signal H3                           
      HALL1             : in  std_logic;  -- Hall Sensor 1 signal input 1
      HALL2             : in  std_logic;  -- Hall Sensor 2 signal input 2
      HALL3             : in  std_logic   -- Hall Sensor 3 signal input 3
      );
  end component tmc603_commutator;

  component tmc603_pwm is               -- top level entity
    port (
      nRST       : in  std_logic;       -- reset
      CLK        : in  std_logic;       -- clock
      PWM_VAL_O  : out std_logic_vector (7 downto 0);  --
      PWM_VAL_I  : in  std_logic_vector (7 downto 0);  --
      PWM_VAL_WR : in  std_logic;       --
      PWM_CLR    : in  std_logic;       -- clear PWM (on change of direction)
      PWM_UP     : in  std_logic;       --   up key (sampled clk/2^16)
      PWM_DN     : in  std_logic;       -- down key (sampled clk/2^16)
      PWM        : out std_logic        -- PWM output
      );
  end component tmc603_pwm;

begin


  syn_p : process(CLK, nRST)
  begin
    if nRST = '0' then
      hall1_ff   <= '0';
      hall2_ff   <= '0';
      hall3_ff   <= '0';
      CLR_ERR_ff <= '1';  -- '1' during reset to clear possible error condition at startup
      SAMPLEx_ff <= '1';
    elsif rising_edge(CLK) then
      hall1_ff   <= HALL1;
      hall2_ff   <= HALL2;
      hall3_ff   <= HALL3;
      CLR_ERR_ff <= '0';
      SAMPLEx_ff <= PB3;  -- toggle SAMPLEx, PB = pressed = 1 = CURx is sampled
                          --                 PB = not pressed = 0 = CURx is hold
    end if;
  end process;

  dir_p : process(CLK, nRST)
  begin
    if nRST = '0' then
      dir_ff      <= '0';
      dir_last_ff <= '0';
      pwm_clr_ff  <= '0';
    elsif rising_edge(CLK) then
      if (PB2 = '1') then
        dir_ff <= '1';
      elsif (PB4 = '1') then
        dir_ff <= '0';
      end if;
      if (dir_last_ff /= dir_ff) then
        pwm_clr_ff <= '0';              -- no CLP_PWM on change of direction
      --pwm_clr_ff <= '1';  -- CLP_PWM on change of direction
      else
        pwm_clr_ff <= '0';
      end if;
      dir_last_ff <= dir_ff;
    end if;
  end process;

  en_dis_p : process(CLK, nRST)
  begin
    -- ENABLE signal is configured as tristate (using ENABLE_TRI_E)
    -- ENABLE_TRI_E = '1' sets ENABLE_TRI_D as output driver, '0' as tristate
    if nRST = '0' then
      ENABLE_TRI_D_ff <= '0';
      ENABLE_TRI_E_ff <= '0';
    elsif rising_edge(CLK) then
      -- if dip-switch is on ('1') --> enable MOSFET bridges of TMC603A
      -- make high impedance, so that TMC603A controls itself
      -- (/err_out and ENABLE are directly connected on the daughter board)
      -- if dip-switch is off ('0') --> disable MOSFET bridges of TMC603A
      if USER_ENN = '0' then
        ENABLE_TRI_D_ff <= '0';         -- tie low = disable
        ENABLE_TRI_E_ff <= '1';         -- no tristate
      else
        ENABLE_TRI_D_ff <= 'Z';         -- tristate = TMC603 controls itself, enabled
        ENABLE_TRI_E_ff <= '0';         -- 
      end if;
    end if;
  end process;

  -- -------------------------------------------------------------------------
  -- Connect Signals to the APB3 Adapter
  -----------------------------------------------------------------------------
  U_RD_DATA(31 downto 8) <= (others => '0');
  U_RD_DATA(7  downto 0) <= pwm_val_o;

  -------------------------------------------------------------------------------
  -- component instances
  -------------------------------------------------------------------------------
  tmc603_pwm_i : tmc603_pwm
    port map(
      nRST       => nRST,
      CLK        => CLK,
      PWM_VAL_O  => pwm_val_o,
      PWM_VAL_I  => (others => '0'),
      PWM_VAL_WR => '0',
      PWM_CLR    => '0',
      PWM_UP     => PB1,
      PWM_DN     => PB5,
      PWM        => pwm); 

  tmc603_commutator_i : tmc603_commutator
    port map(
      nRST              => nRST,
      CLK               => CLK,
      HallFX_EN         => '0',  -- not used yet / external seleced by jumpers
      DualChop          => '0',  -- hallFX requires DualChop='1'
      ChopHS            => '0',  
      OVC               => '0',  -- no over-current detection
      PWM               => pwm,  -- generated in entity 'tmc603_pwm', allternatively, a PWM engine can be used
      DIR               => dir_ff,
      BRAKE             => '0',
      TACHO             => tacho,
      DIR_COM           => dir_com,
      TMC603_BBM_EN     => open,         -- set on PCB, default '1' @ PCB
      TMC603_ENABLE     => open,         -- ENABLE is set below
      TMC603_BH1        => BH1,         -- h/l side bridge control signals
      TMC603_BH2        => BH2,  
      TMC603_BH3        => BH3,  
      TMC603_BL1        => BL1,  
      TMC603_BL2        => BL2,  
      TMC603_BL3        => BL3,  
      TMC603_SAMPLE_123 => open,         
      TMC603_SCCLK      => open,         -- resp. dedicated PWM output, see above
      TMC603_CLR_ERR    => open,         
      TMC603_ERROR_OUT  => '0',
      TMC603_SENSE_HI   => open,         
      TMC603_H1         => HALLFX1,      -- hallFX outputs of TMC603A 
      TMC603_H2         => HALLFX2,      -- hallFX outputs of TMC603A 
      TMC603_H3         => HALLFX3,      -- hallFX outputs of TMC603A 
      HALL1             => hall1_ff,     -- hall sensor 1 of BLDC,
      HALL2             => hall2_ff,     -- hall sensor 2 of BLDC,
      HALL3             => hall3_ff);    -- hall sensor 3 of BLDC

  -----------------------------------------------------------------------------
  -- other signal assignments
  -----------------------------------------------------------------------------
  ENABLE_TRI_D <= ENABLE_TRI_D_ff;  -- ENABLE signal is configured as tristate (using ENABLE_TRI_E)
  ENABLE_TRI_E <= ENABLE_TRI_E_ff;  -- '1' sets D as output driver, '0' = sets D as tristate
  SAMPLEx      <= SAMPLEx_ff;       -- S&H, splitts up into sample1,2,3 (set to high if unused)
  SENSE_HI     <= '1';              -- high resolution measurement
  SCCLK        <= '0';              -- Switched capacitor filter clock input for TMC603A hallFX filters 
  CLR_ERR      <= CLR_ERR_ff;       -- reset error latch (high active), during reset = '1' to clear possible startup error

  LED1_4(0) <= pwm_val_o(0);
  LED1_4(1) <= pwm_val_o(1);
  LED1_4(2) <= pwm_val_o(2);
  LED1_4(3) <= pwm_val_o(3);

end structure;


