-------------------------------------------------------------------------------
--                                                                           --
-- File         : tmc603_pwm.vhd                                             --
--                                                                           --
-- Author(s)    : Dr. Lars Larsson (LL), www.trinamic.com                    --
--                                                                           --
-- Contributors :                                                            --
--                                                                           --
-- Description  : PWM generator for the tmc603_commutator.vhd                --
--                                                                           --
-- Creation     : LL, February 15, 2010                                      --
--                                                                           --
-- History      : LL, February 20, 2010 : set PWM frequency higher           --
--                LL, February 22, 2010 : PWM_CLR slow down (soft stop)      --
--                LL, February 23, 2010 : PWM_VAL_I, PWM_VAL_O, PWM_WR for   -- 
--                                        write and read access from ARM     --
--                                                                           --
-------------------------------------------------------------------------------
-- Hint(s)      :                                                            --
-------------------------------------------------------------------------------
--
-- Copyright (c) 2010, TRINAMIC Motion Control GmbH & Co. KG 
-- All rights reserved.
--
-- Redistribution and use in source and binary forms, with or without 
-- modification, are permitted provided that the following conditions 
-- are met:
--
-- Redistributions of source code must retain the above copyright notice, 
-- this list of conditions and the following disclaimer. Redistributions 
-- in binary form must reproduce the above copyright notice, this list of 
-- conditions and the following disclaimer in the documentation and/or 
-- other materials provided with the distribution. 
--
-- Neither the name of the TRINAMIC nor the names of its contributors may 
-- be used to endorse or promote products derived from this software without 
-- specific prior written permission. 
-- 
-- THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
-- AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
-- IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
-- ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE 
-- LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
-- CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
-- SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
-- INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
-- CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
-- ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
-- THE POSSIBILITY OF SUCH DAMAGE.
--
-------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_signed.all;

entity tmc603_pwm is
  port (
    nRST       : in  std_logic;         -- reset
    CLK        : in  std_logic;         -- clock
    PWM_VAL_O  : out std_logic_vector (7 downto 0);
    PWM_VAL_I  : in  std_logic_vector (7 downto 0);
    PWM_VAL_WR : in  std_logic;
    PWM_CLR    : in  std_logic;         -- clear PWM (on change of direction)
    PWM_UP     : in  std_logic;         --   up key (sampled clk/2^16)
    PWM_DN     : in  std_logic;         -- down key (sampled clk/2^16)
    PWM        : out std_logic          -- PWM output
    );
end tmc603_pwm;

architecture structure of tmc603_pwm is

  signal div_ff     : std_logic_vector (19 downto 0);
  signal pwm_val_ff : std_logic_vector (7 downto 0);
  signal pwm_div_ff : std_logic_vector (4 downto 0);
  signal pwm_cnt_ff : std_logic_vector (7 downto 0);
  signal pwm_ff     : std_logic;
  signal pwm_clr_ff : std_logic;

begin

  updn_p :
  process (nRST, CLK)
  begin
    if nRST = '0' then
      div_ff     <= (others => '0');
      pwm_val_ff <= (others => '0');
    elsif (CLK'event and CLK = '1') then
      if (PWM_VAL_WR = '1') then        -- write access has priority
        pwm_val_ff <= PWM_VAL_I;
      else
        div_ff <= div_ff + 1;
        if (div_ff = x"000000") then
          if (pwm_clr_ff = '1') then
            if (pwm_val_ff /= x"00") then
              pwm_val_ff <= pwm_val_ff - 1;
            end if;
          elsif (PWM_DN = '1') then
            if (pwm_val_ff /= x"00") then
              pwm_val_ff <= pwm_val_ff - 1;
            end if;
          elsif (PWM_UP = '1') then
            if (pwm_val_ff /= x"ff") then
              pwm_val_ff <= pwm_val_ff + 1;
            end if;
          end if;
        end if;
      end if;
    end if;
  end process;


  PWM_VAL_O <= pwm_val_ff;


  pwm_clr_p :
  process (nRST, CLK)
  begin
    if nRST = '0' then
      pwm_clr_ff <= '0';
    elsif (CLK'event and CLK = '1') then
      if (PWM_DN = '1') or (PWM_up = '1') then
        pwm_clr_ff <= '0';
      elsif (pwm_val_ff = x"00") then
        pwm_clr_ff <= '0';
      elsif (PWM_CLR = '1') then  -- soft stop, set pwl_clr_ff flipflop, feeds pwm_val_ff slowly down to x"00" (like dn_ff='1')
        pwm_clr_ff <= '1';
      end if;
    end if;
  end process;


  pwm_p :
  process (nRST, CLK)
    variable pwm_val_limited : std_logic_vector (7 downto 0);
  begin
    if nRST = '0' then
      pwm_div_ff <= (others => '0');
      pwm_cnt_ff <= (others => '0');
      pwm_ff     <= '0';
    elsif (CLK'event and CLK = '1') then
      if (pwm_div_ff = "01000") then    -- set PWM pre-divider via max_count
        pwm_div_ff             <= "00000";
        pwm_cnt_ff             <= pwm_cnt_ff + 1;
        if (('0' & pwm_val_ff) <= ('0' & pwm_cnt_ff)) then
          pwm_ff <= '0';
        else
          pwm_ff <= '1';
        end if;
      else
        pwm_div_ff <= pwm_div_ff + 1;
      end if;
    end if;
  end process;

  PWM <= pwm_ff;

end structure;
