-------------------------------------------------------------------------------
--                                                                           --
-- File         : tmcm_ac_840_top.vhd                                        --
--                                                                           --
-- Author(s)    : Dr. Stephan Kubisch (SK), www.trinamic.com                 --
--                                                                           --
-- Contributors :                                                            --
--                                                                           --
-- Description  : Top level design for the TMCM-AC-840 Motor Control
--                Daughter Card for use with Actel SmartFusion Dev/Eva Kits  --
--                                                                           --
--                The top level design includes 3 main modules: TMC262
--                commutation block, TMC603A commutation block, APB3 bridge
--                module for communication with the MSS. Additionally some glue
--                logic for synchronisation.
--
--                Schematics and layout of the TMCM-AC-840 are available on
--                request. 
--                                                                           --
-- Creation     : SK, May 3, 2010                                            --
--                                                                           --
-- History      : SK, May 5, 2010  : Added the 3 main modules
--                SK, May 12, 2010 : Added USER_ENN to tmc262_commutator_top
--                                   using PB3
--                
--                                                                           --
-------------------------------------------------------------------------------
--                                                                           --
-- Hint(s) : *** DISCONNECT BLDC motor power supply during programming ***   --
--                                                                           --
-------------------------------------------------------------------------------
--
-- Copyright (c) 2010, TRINAMIC Motion Control GmbH & Co. KG 
-- All rights reserved.
--
-- Redistribution and use in source and binary forms, with or without 
-- modification, are permitted provided that the following conditions 
-- are met:
--
-- Redistributions of source code must retain the above copyright notice, 
-- this list of conditions and the following disclaimer. Redistributions 
-- in binary form must reproduce the above copyright notice, this list of 
-- conditions and the following disclaimer in the documentation and/or 
-- other materials provided with the distribution. 
--
-- Neither the name of the TRINAMIC nor the names of its contributors may 
-- be used to endorse or promote products derived from this software without 
-- specific prior written permission. 
-- 
-- THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
-- AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
-- IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
-- ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE 
-- LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
-- CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
-- SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
-- INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
-- CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
-- ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
-- THE POSSIBILITY OF SUCH DAMAGE.
--
-------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

use work.pkg_commutator_components.all;

entity tmcm_ac_840_top is
  
  port (
    FABCLK           : in  std_logic;
    PRESETn          : in  std_logic;   -- active low! M_2_F reset (reset-button)    
    -- LEDs
    LED1             : out std_logic;   -- LEDs currently used
    LED2             : out std_logic;   -- to show lower nibble
    LED3             : out std_logic;   -- of PWM/speed value
    LED4             : out std_logic;
    -- 2 on-board Push-Buttons for various things
    PB1              : in  std_logic;   -- 2 on-board Pushbuttons for various things
    PB2              : in  std_logic;
    -- TMC603A signals
    BL1_603          : out std_logic;
    BH1_603          : out std_logic;
    BH2_603          : out std_logic;
    BL2_603          : out std_logic;
    BH3_603          : out std_logic;
    BL3_603          : out std_logic;
    HALLFX1_603      : in  std_logic;   -- HALLFX signals generated
    HALLFX2_603      : in  std_logic;   -- using the Back-EMF
    HALLFX3_603      : in  std_logic;
    ENABLE_TRI_D_603 : out std_logic;   -- Enable is designed as tristate pin
    ENABLE_TRI_E_603 : out std_logic;   -- TMC603A should be self-regulating (/ERR_OUT & ENABLE are connected)
    SAMPLEx_603      : out std_logic;   -- SAMPLE1,2,3 combined in one signal
    SENSE_HI_603     : out std_logic;   -- Switches current amplifiers to high sensitivity
    SCCLK_603        : out std_logic;   --Switched capacitor filter clock input for hallFX filters
    CLR_ERR_603      : out std_logic;   -- reset of error-ff, active high, clears error condition
    -- TMC262 signals
    CLK_262          : out std_logic;
    SCK_SPI_262      : out std_logic;
    CSN_SPI_262      : out std_logic;
    SDO_SPI_262      : in  std_logic;
    SDI_SPI_262      : out std_logic;
    DIR_262          : out std_logic;
    STEP_262         : out std_logic;
    SG_TST_262       : in  std_logic;
    ENN_262          : out std_logic;
    -- Encoder signals (can be from stepper or BLDC motor)
    ENC_A            : in  std_logic;
    ENC_B            : in  std_logic;
    ENC_N            : in  std_logic;
    -- Hall Sensor Signals from BLDC
    HALL1            : in  std_logic;
    HALL2            : in  std_logic;
    HALL3            : in  std_logic;
    -- APB3 signals to/from MSS
    PENABLE          : in  std_logic;
    PSEL             : in  std_logic;
    PADDR            : in  std_logic_vector(23 downto 0);
    PWRITE           : in  std_logic;
    PWDATA           : in  std_logic_vector(31 downto 0);
    PRDATA           : out std_logic_vector(31 downto 0);
    PREADY           : out std_logic;  -- slave uses PREADY to extent a transfer with wait cycles
    PSLVERR          : out std_logic);

end tmcm_ac_840_top;

architecture arch of tmcm_ac_840_top is


  signal PB1_262, PB2_262, PB4_262, PB5_262 : std_logic;
  signal PB1_603, PB2_603, PB4_603, PB5_603 : std_logic;

  signal PB1_ff, PB2_ff : std_logic;

  signal MRES_262 : std_logic;
  
  signal LEDs_262, LEDs_603, LEDs_OUT : std_logic_vector(3 downto 0);

  signal CLK, nRST : std_logic;

  signal U_WR_REQ  : std_logic;
  signal U_WR_ACK  : std_logic;
  signal U_WR_DATA : std_logic_vector(31 downto 0);
  signal U_WR_ADDR : std_logic_vector(23 downto 0);

  -- Control Registers from MSS
  -- modify at need
  signal CTRL_REG_0_ENN_TMC262 : std_logic_vector( 31 downto 0);  -- 1 bit, default = 1 = off
  signal CTRL_REG_1_ENN_TMC603 : std_logic_vector( 31 downto 0);  -- 1 bit, default = 1 = off
  signal CTRL_REG_2_CTRL_FOCUS : std_logic_vector( 31 downto 0);  -- 1 bit, default = 0 = TMC603A
  signal CTRL_REG_3_CTRL_SELEC : std_logic_vector( 31 downto 0);  -- 4 bits,default = 0 = no control
  signal CTRL_REG_4_SAMPLEx603 : std_logic_vector( 31 downto 0);  -- 1 bit, default = 0 after reset
  signal CTRL_REG_5_CONFIG_SPI : std_logic_vector( 31 downto 0);  -- 1 bit, automatically reset to 0 after 1 cycle

  signal TMC603_PWM_VAL_REG   : std_logic_vector( 31 downto 0);  
  signal TMC262_WC_REG        : std_logic_vector( 31 downto 0);  
  signal TMC262_MR_REG        : std_logic_vector( 31 downto 0);  
  signal TMC262_SPEED_VAL_REG : std_logic_vector( 31 downto 0); 

  
begin  -- arch

  -- default assignments
  CLK  <= FABCLK;
  nRST <= PRESETn;

  LED1 <= LEDs_OUT(0);
  LED2 <= LEDs_OUT(1);
  LED3 <= LEDs_OUT(2);
  LED4 <= LEDs_OUT(3);

  syn_p : process(CLK, nRST)
  begin
    if nRST = '0' then
      PB1_ff <= '0';
      PB2_ff <= '0';      
    elsif rising_edge(CLK) then
      PB1_ff <= not PB1;    -- using "not" because PBs are active low
      PB2_ff <= not PB2;
    end if;
  end process;

  -- LED mux depending on driver block that is currently controled
  led_p : process(CLK, nRST)
  begin
    if nRST = '0' then
      LEDs_OUT <= (others => '0');  
    elsif rising_edge(CLK) then
      -- if 0 then TMC603A is controlled
      if CTRL_REG_2_CTRL_FOCUS(0) = '0' then
        LEDs_OUT <= LEDs_603;
      else -- if '1' then TMC262 is controlled
        LEDs_OUT <= LEDs_262;
      end if;
    end if;
  end process;

  -- select control signals that are currently used
  control_signal_p : process(CLK, nRST)
  begin
    if nRST = '0' then
      
      PB1_262 <= '0';
      PB2_262 <= '0';
      PB4_262 <= '0';
      PB5_262 <= '0';
      PB1_603 <= '0';
      PB2_603 <= '0';
      PB4_603 <= '0';
      PB5_603 <= '0';
      MRES_262 <= '0';
    elsif rising_edge(CLK) then
      -- if TMC603A is controlled
      if CTRL_REG_2_CTRL_FOCUS(0) = '0' then
        case CTRL_REG_3_CTRL_SELEC(1 downto 0) is
          when "01" =>                  -- speed control
            PB1_603 <= PB1_ff;
            PB2_603 <= '0';
            PB4_603 <= '0';
            PB5_603 <= PB2_ff;
          when "10" =>                  -- direction control
            PB1_603 <= '0';
            PB2_603 <= PB1_ff;
            PB4_603 <= PB2_ff;
            PB5_603 <= '0';
          when others =>
            PB1_603 <= '0';
            PB2_603 <= '0';
            PB4_603 <= '0';
            PB5_603 <= '0';
        end case;
        PB1_262 <= '0';
        PB2_262 <= '0';
        PB4_262 <= '0';
        PB5_262 <= '0';
        MRES_262 <= '0';
      else                              -- if TMC262 is controlled
        case CTRL_REG_3_CTRL_SELEC(1 downto 0) is
          when "01" =>                  -- speed control
            PB1_262 <= PB1_ff;
            PB2_262 <= '0';
            PB4_262 <= '0';
            PB5_262 <= PB2_ff;
            MRES_262 <= '0';
          when "10" =>                  -- direction control
            PB1_262 <= '0';
            PB2_262 <= PB1_ff;
            PB4_262 <= PB2_ff;
            PB5_262 <= '0';
            MRES_262 <= '0';
          when "11" =>                  -- microstep resolution control
            PB1_262 <= '0';
            PB2_262 <= PB1_ff;
            PB4_262 <= PB2_ff;
            PB5_262 <= '0';
            MRES_262 <= '1';
          when others =>
            PB1_262 <= '0';
            PB2_262 <= '0';
            PB4_262 <= '0';
            PB5_262 <= '0';
            MRES_262 <= '0';
        end case;
        PB1_603 <= '0';
        PB2_603 <= '0';
        PB4_603 <= '0';
        PB5_603 <= '0';        
      end if;
    end if;
  end process;

  -----------------------------------------------------------------------------
  -- Multiplex and assign write data from APB3 master (MSS) to fabric registers
  --   Write addresses are defined in package
  -----------------------------------------------------------------------------
  apb3_write_mux_p : process (CLK, nRST)
  begin  -- process apb3_write_mux_p
    
    if nRST = '0' then                 -- asynchronous reset (active low)

      -- register reset values
      U_WR_ACK              <= '0';
      CTRL_REG_0_ENN_TMC262 <= (others => '0'); -- the polarity of the ENN signals will be controlled inside each driver block entity
      CTRL_REG_1_ENN_TMC603 <= (others => '0');
      CTRL_REG_2_CTRL_FOCUS <= (others => '0');
      CTRL_REG_3_CTRL_SELEC <= (others => '0');
      CTRL_REG_4_SAMPLEx603 <= (others => '0');
      CTRL_REG_5_CONFIG_SPI <= (others => '0');
      
    elsif rising_edge(CLK) then       -- rising clock edge

      -- if new control data is available check register address
      -- and assign new data to this register
      if U_WR_REQ = '1' then
        case U_WR_ADDR is
          when CTRL_REG_0_ADDR => CTRL_REG_0_ENN_TMC262 <= U_WR_DATA;
          when CTRL_REG_1_ADDR => CTRL_REG_1_ENN_TMC603 <= U_WR_DATA;
          when CTRL_REG_2_ADDR => CTRL_REG_2_CTRL_FOCUS <= U_WR_DATA;
          when CTRL_REG_3_ADDR => CTRL_REG_3_CTRL_SELEC <= U_WR_DATA;
          when CTRL_REG_4_ADDR => CTRL_REG_4_SAMPLEx603 <= U_WR_DATA;
          when CTRL_REG_5_ADDR => CTRL_REG_5_CONFIG_SPI <= U_WR_DATA;
          when others          => null;
        end case;
        U_WR_ACK <= '1';
      else
        U_WR_ACK <= '0';
        -----------------------------------------------------------------------
        -- default unset control bits after 1 clockcycle (so the MSS does not need to do this)
        -----------------------------------------------------------------------
        CTRL_REG_5_CONFIG_SPI <= (others => '0'); -- unset SPI trigger pulse after 1 Clockcycle
      end if;      
    end if;  
  end process apb3_write_mux_p;
  
  -----------------------------------------------------------------------------
  -- Commutation logic for the TMC262 stepper driver
  -- TMC262 is controled via SPI
  -----------------------------------------------------------------------------
  tmc262_commutator_top_1 : tmc262_commutator_top
   port map (
      CLK       => CLK,
      nRST      => nRST,
      PB1       => PB1_262,
      PB2       => PB2_262,
      PB3       => CTRL_REG_5_CONFIG_SPI(0),
      PB4       => PB4_262,
      PB5       => PB5_262,
      LED1_4    => LEDs_262,      
      CLK_262   => CLK_262,
      SCK_SPI   => SCK_SPI_262,
      CSN_SPI   => CSN_SPI_262,
      SDO_SPI   => SDO_SPI_262,
      SDI_SPI   => SDI_SPI_262,
      ENN       => ENN_262,
      DIR       => DIR_262,
      STEP      => STEP_262,
      SG_TST    => SG_TST_262,
      USER_ENN  => CTRL_REG_0_ENN_TMC262(0),
      MRES_SEL  => MRES_262,
      U_RD_DATA_WC => TMC262_WC_REG,
      U_RD_DATA_MR => TMC262_MR_REG,
      U_RD_DATA_SV => TMC262_SPEED_VAL_REG);

  tmc603_commutator_top_1 : tmc603_commutator_top
    port map (
      CLK          => CLK,
      nRST         => nRST,
      LED1_4       => LEDs_603,      
      PB1          => PB1_603,
      PB2          => PB2_603,
      PB3          => CTRL_REG_4_SAMPLEx603(0),
      PB4          => PB4_603,
      PB5          => PB5_603,
      BL1          => BL1_603,
      BH1          => BH1_603,
      BH2          => BH2_603,
      BL2          => BL2_603,
      BH3          => BH3_603,
      BL3          => BL3_603,
      HALL1        => HALL1,
      HALL2        => HALL2,
      HALL3        => HALL3,
      HALLFX1      => HALLFX1_603,
      HALLFX2      => HALLFX2_603,
      HALLFX3      => HALLFX3_603,
      ENABLE_TRI_D => ENABLE_TRI_D_603,
      ENABLE_TRI_E => ENABLE_TRI_E_603,
      SAMPLEx      => SAMPLEx_603,
      SENSE_HI     => SENSE_HI_603,
      SCCLK        => SCCLK_603,
      CLR_ERR      => CLR_ERR_603,
      USER_ENN     => CTRL_REG_1_ENN_TMC603(0),      
      U_RD_DATA    => TMC603_PWM_VAL_REG);

  bridge_APB3_to_userlogic_1 : bridge_APB3_to_userlogic
    port map (
      -- APB3 interface signals to/from APB3 slave port
      PCLK              => CLK,
      PRESETn           => nRST,
      PENABLE           => PENABLE,
      PSEL              => PSEL,
      PADDR             => PADDR,
      PWRITE            => PWRITE,
      PWDATA            => PWDATA,
      PRDATA            => PRDATA,
      PREADY            => PREADY,
      PSLVERR           => PSLVERR,
      -- interface to user logic
      U_WR_REQ          => U_WR_REQ,
      U_WR_ACK          => U_WR_ACK,
      U_WR_ADDR         => U_WR_ADDR,              -- different config values
      U_WR_DATA         => U_WR_DATA,              -- different config values
      CTRL_REG_0        => CTRL_REG_0_ENN_TMC262,
      CTRL_REG_1        => CTRL_REG_1_ENN_TMC603,
      CTRL_REG_2        => CTRL_REG_2_CTRL_FOCUS,
      CTRL_REG_3        => CTRL_REG_3_CTRL_SELEC,
      CTRL_REG_4        => CTRL_REG_4_SAMPLEx603,
      CTRL_REG_5        => CTRL_REG_5_CONFIG_SPI,
      U_RD_DATA_603_PWM => TMC603_PWM_VAL_REG,     -- 603 PWM value
      U_RD_DATA_262_WC  => TMC262_WC_REG,          -- 262 wait cycle count
      U_RD_DATA_262_MR  => TMC262_MR_REG,          -- 262 microstep resolution
      U_RD_DATA_262_SV  => TMC262_SPEED_VAL_REG);  -- 262 speed value


end arch;

